import { setPathValue } from '../utils'
import * as _ from 'lodash'
import * as $utils from '../../../common/utils'
import itemLayout from '../components/item-layout'
import itemHeader from '../components/header'

function broadcast (componentName, eventName, params) {
  this.$children.forEach(child => {
    var name = child.$options.componentName || child.$options.name

    if (name === componentName) {
      child.$emit.apply(child, [eventName].concat(params))
    } else {
      broadcast.apply(child, [componentName, eventName].concat([params]))
    }
  })
}

function nullMerge (targetVal, defalutVal = undefined) {
  return [undefined, null].includes(targetVal) ? defalutVal : targetVal
}

export default {
  components: { itemLayout, itemHeader },
  inject: ['formData', 'loopValue', 'update', 'definitions', 'validateAll'],
  props: {
    formKey: Array,
    type: String,
    title: String,
    description: String,
    showTitle: {
      type: Boolean,
      default: true
    },
    widget: {
      type: Object,
      default: () => ({})
    },
    oneOf: Array,
    anyOf: Array,
    allOf: Array,
    $ref: String,
    default: null,
    mandatory: {
      type: Boolean,
      default: false
    }
  },
  data () {
    return {
      // data: this.loopValue(this.formData, this.formKey),
      utils: $utils,
      data: null,
      key: nullMerge(this.formKey[this.formKey.length - 1], null),
      validation: true,
      errorMsg: ''
    }
  },
  computed: {
    componentId () {
      return `${this.$options.name}-${this._uid}`
    },
    widgetType ({ widget }) {
      return widget.type
    },
    businessCompos ({ $JSONSCHEMA, type }) {
      if ($JSONSCHEMA.businessCompo) {
        return $JSONSCHEMA.businessCompo.filter(x => (x.dataType || []).includes(type)).map(x => `business-${x.type}`)
      }
      return []
    }
  },
  methods: {
    initData () {
      const { type, formData, formKey, loopValue } = this
      if (type !== 'array') {
        let data
        if (type === 'object') {
          let defaultValue = _.cloneDeep(this.default)
          Object.keys(defaultValue || {}).forEach(key => {
            if ($utils.isType(defaultValue[key], 'array')) {
              delete defaultValue[key]
            }
          })
          data = Object.assign(nullMerge(defaultValue, {}), loopValue(formData, formKey) || {})
          if (!Object.keys(data).length && !this.default) data = undefined
        } else {
          data = nullMerge(loopValue(formData, formKey), this.default)
        }
        setPathValue(formData, formKey, data)
        this.data = loopValue(formData, formKey)
      } else {
        this.data = []
      }
    },
    initArrayData () {
      const { type, formData, formKey, loopValue } = this
      if (type === 'array') {
        let data = nullMerge(loopValue(formData, formKey), this.default)

        setPathValue(formData, formKey, data)
        this.data = loopValue(formData, formKey)
      } else if (type === 'object') {
        let defaultValue = _.cloneDeep(this.default)
        Object.keys(defaultValue || {}).forEach(key => {
          if (!$utils.isType(defaultValue[key], 'array')) {
            delete defaultValue[key]
          }
        })
        if (!Object.keys(defaultValue || {}).length) {
          return
        }
        let data = loopValue(formData, formKey) || {}
        Object.keys(defaultValue).forEach(key => {
          if (!data[key]) data[key] = defaultValue[key]
        })
        setPathValue(formData, formKey, data)
        this.data = loopValue(formData, formKey)
        this.broadcast('form-array', 'default-change', [Object.keys(defaultValue)])
      }
    },
    validate () {
      let { loopValue, formData, formKey, mandatory, validateData } = this
      let data = loopValue(formData, formKey)
      let validation = true
      let errorMsg = ''
      if (mandatory && data === undefined) {
        this.validation = false
        this.errorMsg = `This item is required`
        return this.validation
      }
      const { value, msg } = validateData(data)

      validation = value
      errorMsg = msg

      this.validation = validation
      this.errorMsg = errorMsg
      return this.validation
    },
    handleChange (value) {
      console.log('a' + value)
      console.log(this.formKey, value)
      this.update(this.formKey, value)
      this.validate()
      this.dispatch('form-object', 'validate', [])
      this.dispatch('form-array', 'validate', [])
    },
    dispatch (componentName, eventName, params) {
      var parent = this.$parent || this.$root
      var name = parent.$options.componentName

      while (parent && (!name || name !== componentName)) {
        parent = parent.$parent
        if (parent) {
          name = parent.$options.componentName || parent.$options.name
        }
      }
      if (parent) {
        parent.$emit.apply(parent, [eventName].concat(params))
      }
    },
    broadcast (componentName, eventName, params) {
      broadcast.call(this, componentName, eventName, params)
    }
  },
  created () {
    this.initData()
  },
  mounted () {
    this.initArrayData()

    this.$on('validate', params => {
      let validation = this.validate()
      this.dispatch('json-schema-form', 'validate', [validation])
    })
  },
  beforeDestroy () {
  }
}
